package edu.dartmouth.cs.coventrydemo;

import android.graphics.Color;

import androidx.fragment.app.FragmentActivity;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.Polyline;
import com.google.android.gms.maps.model.PolylineOptions;

public class MapsActivity extends FragmentActivity implements OnMapReadyCallback,
        GoogleMap.OnMapClickListener, GoogleMap.OnMapLongClickListener,
        GoogleMap.OnMarkerClickListener {

    private static final String TAG = "MapsActivity";

    private GoogleMap mMap; // Might be null if Google Play services APK is not available.

    TextView tvLocInfo;

    boolean markerClicked;
    PolylineOptions rectOptions;
    Polyline polyline;
    static final LatLng COVENTRY = new LatLng(52.4081, -1.5106);

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_maps);

        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        tvLocInfo = findViewById(R.id.locinfo);
    }


    @Override
    public void onMapReady(GoogleMap googleMap) {

        Log.d(TAG, "onMapReady");

        mMap = googleMap;
        // Uncomment one of the Map types
        mMap.setMapType(GoogleMap.MAP_TYPE_NORMAL);
        mMap.setMapType(GoogleMap.MAP_TYPE_HYBRID);
        mMap.setOnMapClickListener(this);
        mMap.setOnMapLongClickListener(this);
        mMap.setOnMarkerClickListener(this);
        //Move the "camera" instantly to the best city in the world! with a zoom of 15.
        // the desired zoom level is 15. Note the range of 2 (smallest) to 21 (largest)
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(COVENTRY, 15));
        // Moves the map according to the update/zoom with an animation over 2 seconds
        mMap.animateCamera(CameraUpdateFactory.zoomTo(15), 2000, null);
    }


      // GoogleMap.OnMapClickListener interface implemented here
      // Callback interface for when the user taps on the map.
      // Listeners will be invoked on the Android UI thread.

    @Override
    public void onMapClick(LatLng point) {

        Log.d(TAG, "onMapClick");

        tvLocInfo.setText(point.toString());
        // center the camera to the location clicked on map
        mMap.animateCamera(CameraUpdateFactory.newLatLng(point));
        // logic for drawing polyline overlays
        markerClicked = false;
    }


    // GoogleMap.OnMapLongClickListener
    // Callback interface for when the user long presses on the map.

    @Override
    public void onMapLongClick(LatLng point) {

        Log.d(TAG, "onMapLongClick");

        tvLocInfo.setText("New marker added@" + point.toString());
        mMap.addMarker(new MarkerOptions().position(point).title(point.toString()));

        markerClicked = false;
    }

    // GoogleMap.OnMarkerClickListener interface
    // Methods  called when a marker is clicked or tapped

    @Override
    public boolean onMarkerClick(Marker marker) {

        Log.d(TAG, "onMarkerClick");

        if(markerClicked){

            if(polyline != null){
                polyline.remove();
                polyline = null;
            }

            rectOptions.add(marker.getPosition());
            rectOptions.color(Color.RED);
            polyline = mMap.addPolyline(rectOptions);
        }else{
            if(polyline != null){
                polyline.remove();
                polyline = null;
            }

            // A polyline is a list of points, where line segments are drawn between
            // consecutive points. A polyline has the following properties: color, width, etc.
            rectOptions = new PolylineOptions().add(marker.getPosition());
            markerClicked = true;
        }
        return true;
    }
}
